#ifndef PPDATA_H
#define PPDATA_H
#include <qsize.h>
#include <qdebug.h>
#include <qpoint.h>
#include <math.h>

#define MAX_PEAKS 8
#define MAX_ANGLES 120
#define ANGLE_OFFSET 30
#define NOISE_WIDTH 20
#define MAX_DIFF 10
#define MAX_CENTER_TOLERANCE 50
#define HOLE_DISTANCE 2.0
class PPData{
public:
    PPData(uchar *data, QSize size);

    ~PPData();

    double * calcHorProjection();
    int getWidth();
    int getHeight();
    int getNoOfPeaks();
    double *lowPass(double *inData, int dataSize);
    double sigma(double *indata, int inSize);
    double average(double *inData, int inSize);
    double maximum(double *inData, int inSize);
    int findPeaks(double *inData,int inDataSize, int maxPeaks);

    void fillEmittance(bool filtered,bool);
    double * getEmittanceData();
    double * getDataMatrix();
    int *getPeaks();
    void sortPeaks(int *peaks, int size);
    bool peakSanityCheck(int *peaks, int size);
    bool peaksAllZero(int *peaks, int size);
    int *m_peaks,*m_manualPeak;
    bool m_subtractOffset;
    double m_scalingFactor;
    QPoint m_centerPoint;
    void selectAoi(int x,int y,int width, int height);
    void resetAoi();
    double *m_dataMatrix;
    double *m_dataMatrixOrig;

protected:
    int m_height;
    int m_width;
    double *m_horProjection;
    double *m_filteredProjection;
    double *m_verProjection;
    double *m_emittance;

    int m_noOfPeaks;
private:
    int xLow,xHigh;
};

#endif
