#include "ppimage.h"
#include <qpainter.h>
#include <stdlib.h>

PPImage::PPImage(QWidget *parent, const char *) :QLabel(parent)

{
        doDraw = false;
        aoiEnabled = false;
        this->setWhatsThis(QString(tr("the pepperpot image used in projections and emittance plot")));
        this->setToolTip(QString(tr("the current pepperpot image")));
}

PPImage::~PPImage()
{
}

/*!
    \fn PPImage::mouseMoveEvent(QMouseEvent *)
 */
void PPImage::mouseMoveEvent(QMouseEvent *evt)
{
        QPoint pos = evt -> pos();
        x = evt -> x();
        y = evt -> y();
        currentAOI.setWidth(x-currentAOI.x());
        currentAOI.setHeight(y-currentAOI.y());
        update();
        emit mouseXPos(pos.x());
        emit mouseYPos(pos.y()-10);
}

/*!
    \fn PPImage::mousePressEvent(QMouseEvent *)
 */
void PPImage::mousePressEvent(QMouseEvent *evt)
{
        doDraw = true;
        x = evt -> x();
        y = evt -> y();
        currentAOI.setX(x);
        currentAOI.setY(y);
        currentAOI.setWidth(0);
        currentAOI.setHeight(0);
        update();
}

void PPImage::mouseReleaseEvent(QMouseEvent *evt)
{
    if (evt -> button() == Qt::LeftButton)
    {
        if (!aoiEnabled)
            doDraw = false;
        update();
    }
    if (evt -> button() == Qt::RightButton) {
        qDebug() << "Right Button Pressed" << endl;
        emit resetPixmap();
    }
}

void PPImage::setPixmap(QPixmap pixmap)
{
        m_pixmap = pixmap;
        update();
}
/*!
    \fn PPImage::paintEvent(QPaintEvent *)
 */

void PPImage::paintEvent(QPaintEvent *)
{

        QPainter paint(this);

        paint.eraseRect(0,10,width(),height());
        paint.drawPixmap(0,10,m_pixmap);
        if (doDraw) {
                paint.setPen(QColor(Qt::green));
                if (aoiEnabled) {
                    if ((currentAOI.width() > 1) && (currentAOI.height() > 1))
                        paint.drawRect(currentAOI);
                }
                else {
                    paint.drawLine( 0, y ,width(), y);
                    paint.drawLine( x, 0 ,x, height());
            }
        }
}

void PPImage::enableAOI(bool on)
{
    aoiEnabled = on;
    if (!aoiEnabled) {
        doDraw = false;
        update();
    }
}

QRect PPImage::getAOI()
{
    return currentAOI;
}
