#include "projectiondialog.h"
#include <qdebug.h>

ProjectionDialog::ProjectionDialog(QWidget *parent)
    : QDialog(parent), ui_Projection(new Ui::ProjectionDialog)
{
    ui_Projection->setupUi(this);
    this->setWindowTitle(QString(tr("Horizontal Projection")));
    connect(ui_Projection->zoomButton, SIGNAL(toggled(bool)), this, SLOT(enableZoom(bool)));
    m_zoomer = 0;
    m_picker = 0;
    m_zoomEnabled = false;
}
ProjectionDialog::~ProjectionDialog()
{
//    qDebug() << "projectionDialog gone" << endl;
    delete m_picker;
    delete m_zoomer;
}

QwtPlot *ProjectionDialog::getProjectionPlot() {
    return ui_Projection->projectionPlot;
}

void ProjectionDialog::setAverage(double average)
{
    QString dataString;
    dataString.sprintf("%5.2f",average);
    (ui_Projection->meanValueText) -> setText(dataString);
}

void ProjectionDialog::setSigma(double sigma)
{
    QString dataString;
    dataString.sprintf("%5.2f",sigma);
    (ui_Projection -> sigmaText) -> setText(dataString);
    QString whatsThisText = QString(tr("Projection of the pepperpot image. This may include a plot of the raw data, the filtered data as well as a plot of the peak positions"));
    this->setWhatsThis(whatsThisText);
}

QPushButton * ProjectionDialog::getCopyButton()
{
    return ui_Projection->copyPeaksButton;
}
void ProjectionDialog::createZoomer()
{
        m_picker = new QwtPlotPicker(QwtPlot::xBottom, QwtPlot::yLeft,
                                     QwtPicker::PointSelection | QwtPicker::DragSelection,
                                     QwtPicker::CrossRubberBand, QwtPicker::AlwaysOn,
                                     (ui_Projection -> projectionPlot) -> canvas());

        m_picker -> setRubberBandPen(QColor(Qt::green));
        m_picker -> setRubberBand(QwtPicker::CrossRubberBand);
        m_picker -> setTrackerPen(QColor(Qt::black));
        m_picker -> setEnabled(true);

        m_zoomer = new Zoomer(QwtPlot::xBottom, QwtPlot::yLeft,
                               (ui_Projection -> projectionPlot) -> canvas());
        m_zoomer -> setRubberBand(QwtPicker::RectRubberBand);
        m_zoomer -> setRubberBandPen(QColor(Qt::green));
        m_zoomer -> setTrackerMode(QwtPicker::ActiveOnly);
        m_zoomer -> setTrackerPen(QColor(Qt::black));


        m_zoomer -> setEnabled(false);
        m_zoomer -> zoom(0);
        enableZoom(m_zoomEnabled);
}

void ProjectionDialog::enableZoom(bool on)
{
    m_zoomEnabled = on;
    m_picker -> setEnabled(!on);
    m_zoomer -> setEnabled(on);
    m_zoomer -> zoom(0);
}

