#include "qpepperpot.h"
#include "ui_qpepperpot.h"
#include "ppimage.h"
#include "ppdata.h"
#include "assistant.h"

#include <qdebug.h>

QPepperpot::QPepperpot(QWidget *parent)
    : QMainWindow(parent), ui_qpepperpot(new Ui::QPepperpot)
{
    Q_INIT_RESOURCE(iconResources);
    QSettings settings("USPAS","QPepperpot");
    ui_qpepperpot ->setupUi(this);

    assistant = new Assistant;

    m_ppData = 0;
    settingsDialog = 0;
    projectionDialog = 0;
    emittanceDialog = 0;

    m_curve = 0;
    m_filteredCurve = 0;
    m_peakCurve = 0;
    m_emitPlot = 0;

    m_ppImage = new PPImage(ui_qpepperpot -> ppImageBox);
    m_ppImage->setGeometry(QRect(10, 20, 650, 500));

    QString imageFileName = settings.value("imageFileName","").toString();
//    qDebug() << "filename: " << settings.value("imageFilename","");

    if (imageFileName.isEmpty()) {
        if (!setPixmap(QString::fromUtf8(":/icons/images/uspas.png"))) {
            qDebug() << "Default image file: images/uspas.png not found in iconResources, exiting ..." << endl;
            exit(-1);
        }
    }
    else {
//        qDebug() << "Reading filename: " << settings.value("imageFilename","");
        if (!setPixmap(imageFileName))
            if (!setPixmap(QString::fromUtf8(":/icons/images/uspas.png"))) {
            qDebug() << "Default image file: images/uspas.png not found in iconResources, exiting ..." << endl;
            exit(-1);
        }
    }

    setupData();
    readSettings();
/*
 * make sure the check boxes correspond to the settings
 */
    (ui_qpepperpot->actionShow_Filtered)->setChecked(m_enableFilt);
    (ui_qpepperpot->actionShow_Raw)->setChecked(m_enableRaw);
    (ui_qpepperpot->actionShow_Peaks)->setChecked(m_enableShowPeaks);
    (ui_qpepperpot->action_Automatically_find_peaks)->setChecked(m_enableFindPeaks);
    (ui_qpepperpot->actionSubtract_Offset)->setChecked(m_ppData ->m_subtractOffset);
    (ui_qpepperpot->dateTimeEdit)->setDateTime(QDateTime::currentDateTime());

    connect(m_ppImage, SIGNAL(resetPixmap()), this, SLOT(resetAOI()));
    setupActions();


}

QPepperpot::~QPepperpot()
{
    writeSettings();
    delete ui_qpepperpot;
}

void QPepperpot::setupData()
{
        QImage ppImage = m_ppPixmap.toImage();
//        QImage ppImage8bits = ppImage.convertToFormat(QImage::Format_RGB32);
        QSize size = ppImage.size();
        if (ppImage.format() != QImage::Format_RGB32)
            ppImage=ppImage.convertToFormat(QImage::Format_RGB32);

        uchar *pixelData = ppImage.bits();      
//        qDebug() << "size " << size.rheight() << " " << size.rwidth() <<endl;

        if (m_ppData) {
            QPoint center = m_ppData->m_centerPoint;
            double scalingFactor = m_ppData -> m_scalingFactor;
            delete m_ppData;\
            m_ppData = new PPData(pixelData,size);
            m_ppData -> m_scalingFactor = scalingFactor;
            m_ppData -> m_centerPoint = center;
        }
        else
            m_ppData = new PPData(pixelData,size);
}

void QPepperpot::optionsPreferences()
{
    if (settingsDialog==0) {
        settingsDialog = new OptionsDialog(this);
        settingsDialog->show();

        settingsDialog->setImageFileName(m_ppImageFileName);
        settingsDialog->setPeaks(m_ppData->m_manualPeak);        
        settingsDialog->setCenter(m_ppData->m_centerPoint);
        settingsDialog->setScalingFactor(m_ppData->m_scalingFactor);
    }
    connect(settingsDialog, SIGNAL(destroyed()), this, SLOT(settingsDialogDestroyed()));
    connect(settingsDialog, SIGNAL(settingsChangedSignal(bool)), this, SLOT(settingsChanged(bool)));
    connect(settingsDialog->getSortPeaksButton(), SIGNAL(clicked()), this, SLOT(sortPeaks()));

    if (!settingsDialog->isVisible())
        settingsDialog -> show();
    settingsDialog->settingsApply();
}

void QPepperpot::readSettings()
{
//    qDebug() << "Reading settings" << endl;
    QSettings settings("USPAS","QPepperpot");
    m_ppImageFileName = settings.value("imageFilename").toString();
    m_ppData -> m_scalingFactor = settings.value("scalingFactor",37.2).toDouble();
    m_ppData -> m_subtractOffset= settings.value("subtractOffset").toBool();
    m_enableFilt = settings.value("filtered").toBool();
    m_enableFindPeaks = settings.value("findPeaks").toBool();
    m_enableShowPeaks = settings.value("showPeaks").toBool();

    m_enableRaw = settings.value("showRaw").toBool();
    m_enableFilt = settings.value("filtered").toBool();
    int x= settings.value("ppCenterX").toInt();
    int y= settings.value("ppCenterY").toInt();
//    qDebug() << "read center point: " << x << " " << y << endl;
    (m_ppData->m_centerPoint).setX(x);
    (m_ppData->m_centerPoint).setY(y);

    m_ppData->m_scalingFactor = settings.value("scalingFactor").toDouble();
    m_ppData->m_manualPeak[0] = settings.value("peak1").toInt();
    m_ppData->m_manualPeak[1] = settings.value("peak2").toInt();
    m_ppData->m_manualPeak[2] = settings.value("peak3").toInt();
    m_ppData->m_manualPeak[3] = settings.value("peak4").toInt();
    m_ppData->m_manualPeak[4] = settings.value("peak5").toInt();
    m_ppData->m_manualPeak[5] = settings.value("peak6").toInt();
    m_ppData->m_manualPeak[6] = settings.value("peak7").toInt();
    m_ppData->m_manualPeak[7] = settings.value("peak8").toInt();

}


void QPepperpot::writeSettings()
{
//    qDebug() << "Writing settings" << endl;
    QSettings settings("USPAS","QPepperpot");
    settings.setValue("imageFileName",m_ppImageFileName);
    QString fn=settings.value("imageFileName").toString();
//    qDebug() << "imageFileName written, read" << m_ppImageFileName << " " << fn << endl;
    settings.setValue("ppCenterX",m_ppData->m_centerPoint.x());
    settings.setValue("ppCenterY",m_ppData->m_centerPoint.y());
//    qDebug() << "write center x " << m_ppData->m_centerPoint.x() << endl;

    settings.setValue("scalingFactor",m_ppData -> m_scalingFactor);
    settings.setValue("subtractOffset",m_ppData->m_subtractOffset);
    settings.setValue("filtered",m_enableFilt);
    settings.setValue("findPeaks",m_enableFindPeaks);
    settings.setValue("showPeaks",m_enableShowPeaks);
    settings.setValue("showRaw",m_enableRaw);
    //
    // manually defined peaks
    //    
    settings.setValue("peak1",m_ppData->m_manualPeak[0]);
    settings.setValue("peak2",m_ppData->m_manualPeak[1]);
    settings.setValue("peak3",m_ppData->m_manualPeak[2]);
    settings.setValue("peak4",m_ppData->m_manualPeak[3]);
    settings.setValue("peak5",m_ppData->m_manualPeak[4]);
    settings.setValue("peak6",m_ppData->m_manualPeak[5]);
    settings.setValue("peak7",m_ppData->m_manualPeak[6]);
    settings.setValue("peak8",m_ppData->m_manualPeak[7]);
}

void QPepperpot::settingsChanged(bool apply)
{
//    qDebug() << "Change of settings seen" << endl;
    if (!apply) {
        delete settingsDialog;
        settingsDialog = 0;
        return;
    }

    QPoint point = settingsDialog->getCenter();
    m_ppData -> m_centerPoint.setX(point.x());
    m_ppData -> m_centerPoint.setY(point.y());
    m_ppData -> m_scalingFactor = settingsDialog->getScalingFactor();
//    qDebug() << "Change of settings seen" << point.x() << " " << point.y() <<endl;
    int *peaks = settingsDialog->getPeaks();
    for (int i=0; i<MAX_PEAKS;i++) {
        m_ppData->m_manualPeak[i]=peaks[i];
//        qDebug() << "peak(" << i << ") " << peaks[i] << endl;
    }
    updateDialogBoxes();
}


void QPepperpot::setupActions()
{
    /*
     * hook up the actions to their code
     */
    connect(ui_qpepperpot->actionOpen_Image, SIGNAL(triggered()),this,SLOT(fileOpen()));
    connect(ui_qpepperpot->actionQuit, SIGNAL(triggered()),this,SLOT(fileQuit()));
    connect(ui_qpepperpot->actionAbout, SIGNAL(triggered()),this,SLOT(displayAbout()));
    connect(ui_qpepperpot->actionManual,SIGNAL(triggered()),this,SLOT(showDocumentation()));
    connect(ui_qpepperpot->actionWhatsThis, SIGNAL(triggered()),this,SLOT(whatsthis()));
    connect(ui_qpepperpot->actionSave_Image,SIGNAL(triggered()),this,SLOT(fileSaveImage()));
    connect(ui_qpepperpot->actionSave_Hor,SIGNAL(triggered()),this,SLOT(fileSaveProjection()));
    connect(ui_qpepperpot->actionSave_Filtered_Hor,SIGNAL(triggered()),this,SLOT(fileSaveFilteredProjection()));
    connect(ui_qpepperpot->actionSave_Peaks,SIGNAL(triggered()),this,SLOT(fileSavePeaks()));
    connect(ui_qpepperpot->actionSave_Emittance,SIGNAL(triggered()),this,SLOT(fileSaveEmittance()));

    connect(ui_qpepperpot->actionProjections, SIGNAL(triggered()),this,SLOT(showProjection()));
    connect(ui_qpepperpot->action_Emittance, SIGNAL(triggered()),this,SLOT(showEmittance()));

    connect(ui_qpepperpot->actionShow_Filtered, SIGNAL(triggered(bool)),this,SLOT(toggleShowFiltered(bool)));
    connect(ui_qpepperpot->actionShow_Raw, SIGNAL(triggered(bool)),this,SLOT(toggleShowRaw(bool)));
    connect(ui_qpepperpot->actionShow_Peaks, SIGNAL(triggered(bool)),this,SLOT(toggleShowPeaks(bool)));
    connect(ui_qpepperpot->action_Automatically_find_peaks, SIGNAL(triggered(bool)),this,SLOT(toggleFindPeaks(bool)));
    connect(ui_qpepperpot->actionSubtract_Offset, SIGNAL(triggered(bool)),this,SLOT(toggleSubtractOffset(bool)));
    connect(ui_qpepperpot->actionConfigure_Pepperpot, SIGNAL(triggered()),this,SLOT(optionsPreferences()));
 //
 // cursor position
 //
    connect(m_ppImage, SIGNAL(mouseXPos(int)) , ui_qpepperpot->xPosLCD, SLOT(display(int)) );
    connect(m_ppImage, SIGNAL(mouseYPos(int)) , ui_qpepperpot->yPosLCD, SLOT(display(int)) );

    connect(ui_qpepperpot->aoiEnable,SIGNAL(stateChanged(int)),this,SLOT(toggleEnableAOI(int)));
    connect(ui_qpepperpot->aoiSelection,SIGNAL(clicked()),this,SLOT(selectAOI()));

}

void QPepperpot::whatsthis()
{
    QWhatsThis::enterWhatsThisMode();
}

void QPepperpot::settingsDialogDestroyed()
{
        settingsDialog = 0;
 //       qDebug() << "settings dialog destroyed" << endl;
}

void QPepperpot::fileOpen()
{
    QString fileName = QFileDialog::getOpenFileName(this,
     tr("Open Image"), "images", tr("Image Files (*.png *.jpg *.bmp)"));

//    qDebug() << "Filename" << fileName << endl;
    m_ppImageFileName = fileName;

    QPixmap pixmap(fileName);

    if (pixmap.isNull()) {
        QMessageBox msgBox;
        msgBox.setText(fileName + tr(" is not a valid pixmap file"));
        msgBox.setIcon(QMessageBox::Critical);
        msgBox.exec();
        return;
    }
    if (setPixmap(fileName))
        m_ppPixmap = pixmap;
    else
        return;
    setupData();
    (ui_qpepperpot->dateTimeEdit)->setDateTime(QDateTime::currentDateTime());
    if (projectionDialog)
        if(projectionDialog->isVisible()) {
            showProjection();
//            qDebug() << "Updating projection" << endl;
        }

        if (emittanceDialog) {
            delete emittanceDialog;
            emittanceDialog = 0;
            showEmittance();
        }
        ui_qpepperpot->aoiEnable->setChecked(false);
}
void QPepperpot::showDocumentation()
{
     assistant->showDocumentation("index.html");
}
void QPepperpot::showProjection()
{
        double *horProjection;
        double *filteredProjection=0;

        QString numberString;
        if (projectionDialog==0) {
//            qDebug() << " Create new projection dialog" << endl;
            projectionDialog = new ProjectionDialog(this);
            projectionDialog->show();
            connect(projectionDialog,SIGNAL(destroyed(QObject*)), this ,SLOT(projectionDialogDestroyed()));
            connect(projectionDialog->getCopyButton(), SIGNAL(clicked()), this, SLOT(copyToManualPeaks()));
        }
        if(!projectionDialog->isVisible()) {
            projectionDialog->show();
//            qDebug() << "ProjectionDialog was invisible" << endl;
        }
        int width = m_ppData -> getWidth();
//        qDebug() << "width:" << width << endl;

        double *xValues = new double[width];
        double *xValPtr;
        double average,sigma,max;
        xValPtr = xValues;
        for (int i=0; i<width; i++)
                *xValPtr++ = (double) i;
        QwtPlot *horPlot = projectionDialog -> getProjectionPlot();

        horProjection = m_ppData -> calcHorProjection();

        for (int i=0;i<width;i++)
                horProjection[i] = horProjection[i];

        horPlot -> setAxisScale(QwtPlot::xBottom,0.0,(double)width);

        max = m_ppData -> maximum(horProjection,width);
        //
        // add 10%, just in case
        //

        max = max + max/10;
        horPlot -> setAxisScale(QwtPlot::yLeft,0.0,max);

        if (m_curve)
        {
                delete m_curve;
                m_curve = 0;
        }

        if (!m_enableFilt || m_enableRaw)
        {
                m_curve = new QwtPlotCurve("Hor Projection");
                m_curve -> setData(xValues,horProjection,width);
                m_curve -> setPen(QPen(Qt::blue));
                m_curve -> attach(horPlot);
                average = m_ppData -> average(horProjection,NOISE_WIDTH);
 //               m_view -> setAverage(average);
 //               sigma = m_ppData -> sigma(horProjection,NOISE_WIDTH);
 //               m_view -> setSigma(sigma);
        }

        if (m_filteredCurve)
        {
                delete m_filteredCurve;
                m_filteredCurve = 0;
        }
        if (m_enableFilt)
        {
                m_filteredCurve = new QwtPlotCurve("Low Pass filtered Projection");
                filteredProjection = m_ppData -> lowPass(horProjection,width);
                m_filteredCurve -> setData(xValues,filteredProjection,width);
                m_filteredCurve -> setPen(QPen(Qt::red));
                m_filteredCurve -> attach(horPlot);
                average = m_ppData -> average(filteredProjection,NOISE_WIDTH);
                projectionDialog -> setAverage(average);
                sigma = m_ppData -> sigma(filteredProjection,NOISE_WIDTH);
                projectionDialog -> setSigma(sigma);
        }

        if (m_peakCurve)
        {
                delete m_peakCurve;
                m_peakCurve = 0;
        }
        if (m_enableFindPeaks)
        {
                printf("peak find enabled\n");
                int *peaks;
//                double getNoiseSigma();
//                double getAverageNoise();
                int noOfPeaks;
                if (m_enableFilt)
                        noOfPeaks = m_ppData -> findPeaks(filteredProjection,width,20);
                else
                        noOfPeaks = m_ppData -> findPeaks(horProjection,width,20);

                if (noOfPeaks > 0) {
                        peaks = m_ppData -> getPeaks();

                        m_peakCurve = new QwtPlotCurve("Peaks");
                        double *peakData = new double[width];
                        double *peakDataPtr = peakData;
                        int *peakPtr = peaks;
                        for (int i=0; i<width; i++) {
                                if (i==*peakPtr) {
                                        if (m_enableFilt)
                                                *peakDataPtr++ = filteredProjection[i];
                                        else
                                                *peakDataPtr++ = horProjection[i];
                                        peakPtr++;
                                } else
                                        *peakDataPtr++ = 0.0;
                        }
                        //
                        // show the peaks only if asked to
                        //
                        if (m_enableShowPeaks) {
                                m_peakCurve -> setData(xValues,peakData,width);
                                m_peakCurve -> setPen(QPen(Qt::black));
                                m_peakCurve -> attach(horPlot);
                        }
                }
        }
        if ((m_enableShowPeaks) && (!m_enableFindPeaks))
        {
                int *peaks;
                int noOfPeaks = 0;
                if (!m_ppData -> peaksAllZero(m_ppData -> m_manualPeak,MAX_PEAKS)) {
                        peaks = m_ppData -> m_manualPeak;
                        m_ppData -> sortPeaks(peaks,MAX_PEAKS);
                        for (int i=0; i < MAX_PEAKS; i++)
                                if (peaks[i] != 0)
                                        noOfPeaks++;
                        m_peakCurve = new QwtPlotCurve("Peaks");
                        double *peakData = new double[width];
                        double *peakDataPtr = peakData;
                        int *peakPtr = peaks;
                        //
                        // point to the first non null entry
                        //
                        for (int i=0;*peakPtr == 0; i++,peakPtr++) {};
                        for (int i=0; i<width; i++) {
                                if (i==*peakPtr) {
                                        if (m_enableFilt)
                                                *peakDataPtr++ = filteredProjection[i];
                                        else
                                                *peakDataPtr++ = horProjection[i];
                                        peakPtr++;
                                } else
                                        *peakDataPtr++ = 0.0;
                        }
                        m_peakCurve -> setData(xValues,peakData,width);
                        m_peakCurve -> setPen(QPen(Qt::black));
                        m_peakCurve -> attach(horPlot);
                }
        }
       if (projectionDialog -> m_zoomer) {
            delete projectionDialog -> m_zoomer;
            delete projectionDialog -> m_picker;
        }
        projectionDialog -> createZoomer();
        horPlot -> replot();

}

void QPepperpot::projectionDialogDestroyed()
{
        projectionDialog = 0;
        m_curve = 0;
        m_filteredCurve = 0;
        m_peakCurve = 0;
 //       qDebug() << "projection destroyed" << endl;
}

void QPepperpot::showEmittance()
{
//        qDebug() <<"showEmittance" << endl;
        if (emittanceDialog == 0) {
            emittanceDialog = new EmittanceDialog(this);
//            qDebug() << "Creating new emittance dialog" << endl;
        }
        //
        // check if the projection plot is on display
        // if not, ask for the calculations, just in case...
        //
        int noOfPeaks;
        int *peaks;
        double *horProjection,*filteredProjection;
        double max;
        QString numberString;
        QMessageBox msgBox;

        int width = m_ppData -> getWidth();

        horProjection = m_ppData -> calcHorProjection();
        for (int i=0;i<width;i++)
                horProjection[i] = horProjection[i]/1000.0;
        filteredProjection = m_ppData -> lowPass(horProjection,width);

        if (m_enableFindPeaks)
        {
                if (m_enableFilt) {
                        filteredProjection = m_ppData -> lowPass(horProjection,width);
                        noOfPeaks = m_ppData -> findPeaks(filteredProjection,width,MAX_PEAKS);
//                        qDebug() <<"filtered no of peaks: " << noOfPeaks << endl;

                        if (noOfPeaks != MAX_PEAKS) {
                                emittanceDialog->hide();
                                if (noOfPeaks < 0)
                                        numberString = "more than 8";
                                else
                                numberString = QString::number(noOfPeaks);
                                msgBox.setText(tr("Wrong number peaks (") + numberString + tr(") found\nPlease define peaks manually"));
                                msgBox.setIcon(QMessageBox::Critical);
                                msgBox.exec();
                                return;
                            }
                } else {
                        noOfPeaks = m_ppData -> findPeaks(horProjection,width,MAX_PEAKS);
                        if (noOfPeaks != MAX_PEAKS) {
                                emittanceDialog->hide();
                                if (noOfPeaks < 0)
                                        numberString = "more than 8";
                                else
                                        numberString = QString::number(noOfPeaks);
                                msgBox.setText(tr("Wrong number peaks (") + numberString + tr(") found\nPlease define peaks manually or try to filter the data"));
                                msgBox.setIcon(QMessageBox::Critical);
                                msgBox.exec();
                                return;
                        }
                }

        }

        //
        // some more checks on manual definition of peaks
        //
//        if (!enableFindPeaks) { 									// m_enableFindPeaks
        else
        {
//                qDebug() << "not enable find peaks" << endl;

                peaks = m_ppData -> m_manualPeak;
                m_ppData -> sortPeaks(peaks,MAX_PEAKS);
                if (peaks[0] == 0) {
                        emittanceDialog->hide();
                        msgBox.setText(tr("Not all peaks have been defined!\nPlease correct"));
                        msgBox.setIcon(QMessageBox::Critical);
                        msgBox.exec();
                        return;
                }

                if (!(m_ppData -> peakSanityCheck(peaks, MAX_PEAKS))) {
                        emittanceDialog->hide();
                        msgBox.setText(tr("Peaks look strange!\nPlease correct"));
                        msgBox.setIcon(QMessageBox::Critical);
                        msgBox.exec();
                        projectionDialog->hide();
                        return;
                }

       }

       //
       // make sure the center point looks easonable
       //

        if (m_ppData->m_scalingFactor < 1.0) {
           emittanceDialog->hide();
           msgBox.setText(tr("The scaling factor is wrongly set, clease correct"));
           msgBox.setIcon(QMessageBox::Critical);
           msgBox.exec();
           return;
        }
/*
        if ((midPoint < width/2 - MAX_CENTER_TOLERANCE) || (midPoint > width/2 + MAX_CENTER_TOLERANCE)) {
           emittanceDialog->hide();
           msgBox.setText(tr("Center point is out of tolerance limits\nPlease correct center point"));
           msgBox.setIcon(QMessageBox::Critical);
           msgBox.exec();
           return;
        }
*/
//        qDebug() << "Filling emittance plot" << endl;
        m_ppData -> fillEmittance(m_enableFilt,m_enableFindPeaks);
        if (m_enableFindPeaks)
            max = m_ppData -> maximum(filteredProjection,width);
        else
            max = m_ppData -> maximum(horProjection,width);
//        qDebug() << "maximum: " << max << endl;

        connect(emittanceDialog, SIGNAL(destroyed()), this, SLOT(emittanceDialogDestroyed()));

        if (m_emitPlot == 0)
                m_emitPlot = new Qwt3D::SurfacePlot(emittanceDialog);

        m_emitPlot -> setTitle("The final Emittance Plot");

        EmittanceFunction emittanceFunction(m_emitPlot);
        emittanceFunction.setMesh(MAX_PEAKS,MAX_ANGLES);
        emittanceFunction.setDomain(0.0,MAX_PEAKS-1,0.0,MAX_ANGLES-1);
        emittanceFunction.setMinZ(0.0);

        emittanceFunction.setEmitData(m_ppData -> getEmittanceData());
        emittanceFunction.create();

//        qDebug() << "max: " << max << endl;
        m_emitPlot -> setRotation(30,0,15);
        m_emitPlot -> setScale(8,1,PLOT_HEIGHT/max);
        m_emitPlot -> setShift(0.15,0,0);
        m_emitPlot -> setZoom(0.9);

        for (unsigned i=0; i!=m_emitPlot -> coordinates()->axes.size(); ++i)
        {
                m_emitPlot -> coordinates()->axes[i].setMajors(7);
                m_emitPlot -> coordinates()->axes[i].setMinors(4);
        }


        m_emitPlot -> coordinates()->axes[X1].setLabelString("x-axis");
        m_emitPlot -> coordinates()->axes[Y1].setLabelString("y-axis");

        m_emitPlot -> setCoordinateStyle(BOX);
        m_emitPlot -> resize(800,600)	;
        m_emitPlot -> updateData();
        m_emitPlot -> updateGL();

//        (emittanceDialog) -> setAttribute( Qt::WA_DeleteOnClose );
        emittanceDialog -> show();

}
bool QPepperpot::setPixmap(const QString pixmapName)
{

        m_ppPixmap = QPixmap(pixmapName);
        if (m_ppPixmap.isNull()) {
            QMessageBox msgBox;
            msgBox.setText(tr("Pixmap File Name: ") + pixmapName + tr (" is invalid"));
            msgBox.setIcon(QMessageBox::Critical);
            msgBox.exec();;
            return false;
        }

        m_ppImage -> setPixmap(m_ppPixmap);

        (ui_qpepperpot -> ppImageBox) -> setTitle(tr("Pixmap file: ") + pixmapName);
        return true;
}

void QPepperpot::emittanceDialogDestroyed()
{
        emittanceDialog = 0;
        m_emitPlot = 0;
//        qDebug() << "emittance dialog destroyed" << endl;
}

void QPepperpot::displayAbout()
{
    QString titleString(tr("About QPepperpot"));
    QString aboutString1(tr("QPepperpot is a Qt-4.5 application for evaluation of pepperpot images\n"));
    QString aboutString2(tr("The program has been written for USPAS,\n"));
    QString aboutString3(tr("the US accelerator school, dedicated to beam diagnostics.\n"));
    QString aboutString4(tr("Albuquerque 22.6 - 26.6\n"));
    QString authorString(tr("Author: Uli Raich, CERN BE/BI (uli.raich@cern.ch)"));
    QMessageBox::about(this,titleString,aboutString1+aboutString2+aboutString3+aboutString4+authorString);
}

void QPepperpot::toggleShowFiltered(bool on)
{
        if (on)
                m_enableFilt = true;
        else
                m_enableFilt = false;
//        if (m_enableFilt)
//            qDebug() << "filtering switched on" << endl;
//        else
//            qDebug() << "filtering switched off" << endl;
/*
        Settings::setFiltered(on);
        Settings::self()->writeConfig();
*/

        updateDialogBoxes();
}

void QPepperpot::toggleShowRaw(bool on)
{
        if (on)
                m_enableRaw = true;
        else
                m_enableRaw = false;

//        Settings::setRaw(on);
//        Settings::self()->writeConfig();

        updateDialogBoxes();
}

void QPepperpot::toggleFindPeaks(bool on)
{
//        qDebug() << "toggle find peaks" << endl;
        if (on)
                m_enableFindPeaks = true;
        else
                m_enableFindPeaks = false;

//        Settings::setFindPeaks(on);
//        Settings::self() -> writeConfig();

        updateDialogBoxes();
}


void QPepperpot::toggleSubtractOffset(bool on)
{

        if (on)
                m_ppData -> m_subtractOffset = true;
        else
                m_ppData -> m_subtractOffset = false;

//        Settings::setSubtractOffset(on);
//        Settings::self() -> writeConfig();
        updateDialogBoxes();
 }

void QPepperpot::toggleShowPeaks(bool on)
{
//        qDebug() << "toggle show peaks" << endl;
        if (on)
                m_enableShowPeaks = true;
        else
        {
                m_enableShowPeaks = false;

        }

        updateDialogBoxes();
}
void QPepperpot::toggleEnableAOI(int )
{
    m_ppImage->enableAOI(ui_qpepperpot->aoiEnable->isChecked());
}

void QPepperpot::selectAOI()
{
    QRect currentAOI;
    if (!ui_qpepperpot->aoiEnable->isChecked())
        return;
    else {
        currentAOI = m_ppImage->getAOI();
        if (currentAOI.width() < 200) {
            QMessageBox msgBox;
            msgBox.setText(tr("AOI is too small, not taken into account"));
            msgBox.setIcon(QMessageBox::Critical);
            msgBox.exec();
            return;
        }
        m_ppData->selectAoi(currentAOI.x(),currentAOI.y(),
                            currentAOI.width()+currentAOI.x(),currentAOI.height()+currentAOI.y());

    QImage pImage = m_ppPixmap.toImage();
    if (pImage.format() != QImage::Format_RGB32)
        pImage.convertToFormat(QImage::Format_RGB32);

    uchar *pixelData = pImage.bits();

    for (int i=0; i < m_ppData->getWidth()*m_ppData->getHeight(); i++) {
        pixelData[4*i] = (unsigned int)m_ppData -> m_dataMatrix[i];
        pixelData[4*i+1] = (unsigned int)m_ppData -> m_dataMatrix[i];
        pixelData[4*i+2] = (unsigned int)m_ppData -> m_dataMatrix[i];
        pixelData[4*i+3] = 0xff;
    }

    QPixmap newPixmap = QPixmap::fromImage(pImage);
    m_ppImage -> setPixmap(newPixmap);
//        qDebug() << "select new AOI" << endl;
    }
    updateDialogBoxes();
}

void QPepperpot::resetAOI()
{
    QRect currentAOI;
    if (!ui_qpepperpot->aoiEnable->isChecked())
        return;
    m_ppData->resetAoi();
    QImage pImage = m_ppPixmap.toImage();
    uchar *pixelData = pImage.bits();

    for (int i=0; i < m_ppData->getWidth()*m_ppData->getHeight(); i++) {
        pixelData[4*i] = (unsigned int)m_ppData -> m_dataMatrix[i];
        pixelData[4*i+1] = (unsigned int)m_ppData -> m_dataMatrix[i];
        pixelData[4*i+2] = (unsigned int)m_ppData -> m_dataMatrix[i];
        pixelData[4*i+3] = 0xff;
    }

    QPixmap newPixmap = QPixmap::fromImage(pImage);
    m_ppImage -> setPixmap(newPixmap);
//    qDebug() << "reset AOI" << endl;
    updateDialogBoxes();

}
void QPepperpot::updateDialogBoxes()
{
        if (projectionDialog)
            if(projectionDialog->isVisible()) {
                showProjection();
//                qDebug() << "Updating projection" << endl;
            }
        if (emittanceDialog)
            if (emittanceDialog->isVisible()) {
                showEmittance();
//                qDebug() << "Updating projection" << endl;
            }
 }

void QPepperpot::copyToManualPeaks()
{
        int noOfPeaks = m_ppData -> getNoOfPeaks();
        int *peaks = m_ppData -> getPeaks();
//        qDebug() << noOfPeaks <<" peaks to copy" << endl;

        if (!(noOfPeaks > 0))
        {
            QMessageBox msgBox;
            msgBox.setText(tr("Peak finding gave no valid results\nPlease define peaks manually"));
            msgBox.setIcon(QMessageBox::Critical);
            msgBox.exec();
        }

        for (int i=0; i<noOfPeaks; i++)
                m_ppData -> m_manualPeak[i] = peaks[i];
        if (noOfPeaks < MAX_PEAKS)
                for (int i=noOfPeaks; i<MAX_PEAKS; i++)
                        m_ppData -> m_manualPeak[i] = 0;

        for (int i=0;i<8;i++)
//                qDebug() << "Copy peaks: " << i << ", " << peaks[i] << endl;

        if (settingsDialog) {
            settingsDialog->setPeaks(peaks);
            settingsDialog->settingsApply();
        }
        return;
}

void QPepperpot::sortPeaks()
{
    m_ppData->sortPeaks(m_ppData->m_manualPeak,MAX_PEAKS);
//    for (int i=0; i<MAX_PEAKS;i++)
//        qDebug() << "peak " << i << ": " << m_ppData->m_manualPeak[i];

    settingsDialog->setPeaks(m_ppData->m_manualPeak);
    settingsDialog->settingsApply();
}

void QPepperpot::fileSaveProjection()
{
        // this slot is called whenever the File->Save Hor menu is selected
    QString fileName = QFileDialog::getSaveFileName(this,
        tr("Save Projection"), "images", tr("Projections (*.dat)"));
//    qDebug() << "Filename: " << fileName << endl;

    QFile outputFile(fileName);
    if (!outputFile.open(QIODevice::WriteOnly)) {
        QMessageBox msgBox;
        msgBox.setText(tr("Cannot open File") );
        msgBox.setIcon(QMessageBox::Critical);
    }
    QTextStream textStream(&outputFile);
    double *projection = m_ppData -> calcHorProjection();
    QString dataString;
    int width = m_ppData -> getWidth();
    for (int i=0; i<width; i++) {
         dataString.sprintf("%f\n",projection[i]);
         textStream << dataString;
    }

    outputFile.close();
    return;
}

void QPepperpot::fileSaveFilteredProjection()
{
        // this slot is called whenever the File->Save Hor menu is selected
    QString fileName = QFileDialog::getSaveFileName(this,
        tr("Save Projection"), "images", tr("Projections (*.dat)"));
//    qDebug() << "Filename: " << fileName << endl;

    QFile outputFile(fileName);
    if (!outputFile.open(QIODevice::WriteOnly)) {
        QMessageBox msgBox;
        msgBox.setText(tr("Cannot open File") );
        msgBox.setIcon(QMessageBox::Critical);
    }
    QTextStream textStream(&outputFile);
    double *projection = m_ppData -> calcHorProjection();
    int width = m_ppData -> getWidth();
    double *filteredProjection = m_ppData->lowPass(projection,width);
    QString dataString;
    for (int i=0; i<width; i++) {
         dataString.sprintf("%f\n",filteredProjection[i]);
         textStream << dataString;
    }

    outputFile.close();
    return;
}

void QPepperpot::fileSaveEmittance()
{
        // this slot is called whenever the File->Save Emittance menu is selected
    QString fileName = QFileDialog::getSaveFileName(this,
        tr("Save Emittance"), "images", tr("emittance (*.dat)"));
//    qDebug() << "Filename: " << fileName << endl;

    QFile outputFile(fileName);
    if (!outputFile.open(QIODevice::WriteOnly)) {
        QMessageBox msgBox;
        msgBox.setText(tr("Cannot open File") );
        msgBox.setIcon(QMessageBox::Critical);
    }
    QTextStream textStream(&outputFile);
    double *emittance = m_ppData->getEmittanceData();
    QString dataString;
    for (int i=0; i<MAX_PEAKS; i++) {
        for(int j=0; j<MAX_ANGLES; j++) {
         dataString.sprintf("%f ",emittance[i*MAX_ANGLES+j]);
         textStream << dataString;
        }
        textStream << endl;
    }

    outputFile.close();
    return;
}

void QPepperpot::fileSaveImage()
{
        // this slot is called whenever the File->Save Image menu is selected
    QString fileName = QFileDialog::getSaveFileName(this,
        tr("Save Image"), "images", tr("image (*.dat)"));
//    qDebug() << "Filename: " << fileName << endl;

    QFile outputFile(fileName);
    if (!outputFile.open(QIODevice::WriteOnly)) {
        QMessageBox msgBox;
        msgBox.setText(tr("Cannot open File") );
        msgBox.setIcon(QMessageBox::Critical);
    }
    QTextStream textStream(&outputFile);
    double *dataMatrix = m_ppData->getDataMatrix();
    int width = m_ppData->getWidth();
    int height=m_ppData->getHeight();
    QString dataString;
    for (int i=0; i<width; i++) {
        for(int j=0; j<height; j++) {
         dataString.sprintf("%f ",dataMatrix[i*width+j]);
         textStream << dataString;
        }
        textStream << endl;
    }

    outputFile.close();
    return;
}

void QPepperpot::fileSavePeaks()
{
    // this slot is called whenever the File->Save Image menu is selected
    QString fileName = QFileDialog::getSaveFileName(this,
        tr("Save Peaks"), "images", tr("peaks (*.dat)"));
//    qDebug() << "Filename: " << fileName << endl;

    QFile outputFile(fileName);
    if (!outputFile.open(QIODevice::WriteOnly)) {
        QMessageBox msgBox;
        msgBox.setText(tr("Cannot open File") );
        msgBox.setIcon(QMessageBox::Critical);
    }
    QTextStream textStream(&outputFile);

    int *peaks;
    if (m_enableFindPeaks)
        peaks = m_ppData->getPeaks();
    else
        peaks = m_ppData->m_manualPeak;
    for (int i=0; i<MAX_PEAKS; i++) {
         textStream << peaks[i] << endl;
    }
    outputFile.close();
    return;
}

void QPepperpot::fileQuit()
{
    QCoreApplication::quit();
}
